/*********************************************************************
 * package.c - Display package window.
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2005 David Rosal <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/

#include "gpaco.h"


/* Remove the row of the package with id = pkg_id from the tree view */
static gboolean remove_row(GtkTreeModel *model, __UNUSED GtkTreePath *path,
                           GtkTreeIter *iter, gpointer pkg_id)
{
	gint id;
	
	gtk_tree_model_get(model, iter, COL_ID, &id, -1);
	
	if (GPOINTER_TO_INT(pkg_id) == id) {
		gtk_list_store_remove(GTK_LIST_STORE(model), iter);
		return TRUE;
	}
	return FALSE;
}


gboolean destroy_pkg(pkg_t *pkg)
{
	g_return_val_if_fail(pkg != NULL, TRUE);
	
	if (pkg->window) {
		gtk_widget_destroy(pkg->window);
		pkg->window = NULL;
	}
	if (!pkg->log) {
		gtk_tree_model_foreach(_model, remove_row, GINT_TO_POINTER(pkg->id));
		free_pkg_data(pkg, NULL);
		_pkgs = g_slist_remove(_pkgs, (gpointer)pkg);
	}
	
	return TRUE;
}


static void gp_nbpage(GtkWidget *notebook, GtkWidget *page, const gchar *title,
                      const gchar *stock_id)
{
	GtkWidget *label, *image, *hbox;

	label = gtk_label_new(title);
	image = gtk_image_new_from_pixbuf(gtk_widget_render_icon(label, stock_id,
	  GTK_ICON_SIZE_MENU, NULL));
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show_all(hbox);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, hbox);
}


void pkg_window_new(pkg_t *pkg, gint page_id)
{
	GtkWidget *nb;

	g_return_if_fail(pkg != NULL);

	if (pkg->window) {
		/* If the window is already built, simply set the proper page */
		nb = g_object_get_data(G_OBJECT(pkg->window), "notebook");
		g_return_if_fail(nb != NULL);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), page_id);
		gtk_window_present(GTK_WINDOW(pkg->window));
		return;
	}
	
	pkg->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(pkg->window), pkg->name); 
	gtk_window_set_default_size(GTK_WINDOW(pkg->window), 400, 500);
	g_signal_connect_swapped(pkg->window, "delete-event", G_CALLBACK(destroy_pkg), pkg);
	
	nb = gtk_notebook_new();	
	gp_nbpage(nb, files_page(pkg), _("Files"), GTK_STOCK_JUSTIFY_FILL);
	gp_nbpage(nb, properties_page(pkg), _("Properties"), GTK_STOCK_PROPERTIES);
	gp_nbpage(nb, remove_page(pkg), _("Remove"), GTK_STOCK_DELETE);
	
	g_object_set_data(G_OBJECT(pkg->window), "notebook", nb);
	gtk_container_add(GTK_CONTAINER(pkg->window), nb);
	
	gtk_widget_show_all(pkg->window);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), page_id);
}


/* vim: set ts=4 */
