/*********************************************************************
 * preferences.c : The "Preferences" dialog.
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2005 David Rosal <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/

#include "gpaco.h"


static gint block_size;
static int opt;
static GtkWidget *prefs_dialog;


static gboolean update_main_window(GtkTreeModel *model, __UNUSED GtkTreePath *path,
								   GtkTreeIter *iter, __UNUSED gpointer data)
{
	gfloat size;
	gint date;
	
	gtk_tree_model_get(model, iter, COL_SIZE, &size, -1);
	gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_SIZE, size, -1);
	gtk_tree_model_get(model, iter, COL_MISSING_SIZE, &size, -1);
	gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_MISSING_SIZE, size, -1);
	gtk_tree_model_get(model, iter, COL_DATE, &date, -1);
	gtk_list_store_set(GTK_LIST_STORE(model), iter, COL_DATE, date, -1);
	
	return FALSE;
}


static void set_blksize(GtkWidget *button, gpointer size)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
		block_size = GPOINTER_TO_INT(size);
}


static void set_date_format(GtkWidget *button, gpointer data)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
		opt |= GPOINTER_TO_INT(data);
	else
		opt &= ~(GPOINTER_TO_INT(data));
}


static void set_file_sizes(pkg_t *pkg, __UNUSED gpointer data)
{
	g_return_if_fail(pkg != NULL);
	if (pkg->window)
		files_treeview_update(pkg);
}


static void pref_response(__UNUSED GtkDialog *dialog, gint response_id, __UNUSED gpointer data)
{
	if (response_id == GTK_RESPONSE_OK) {
		/* Size units */
		if (_blksize != block_size) {
			_blksize = block_size;
			g_slist_foreach(_pkgs, (GFunc)set_file_sizes, NULL);
		}
		_opt = opt;
		gtk_tree_model_foreach(_model, update_main_window, NULL);
	}
	
	gtk_widget_destroy(prefs_dialog);
	REFRESH_GUI;
}		


static void pref_dialog_destroy()
{
	g_return_if_fail(prefs_dialog != NULL);
	gtk_widget_destroy(prefs_dialog);
	prefs_dialog = NULL;
}


static void switch_percent()
{
	opt ^= MISS_SIZE_AS_PERCENT;
}


static GtkWidget *gp_radio_button(const gchar *label, GtkWidget *ref, gboolean active,
                                  GtkWidget *box, GCallback cb, gpointer cb_data)
{
	GtkWidget *button;
	
	if (ref)
		button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(ref), label);
	else
		button = gtk_radio_button_new_with_mnemonic(NULL, label);
	
	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);

	if (cb)
		g_signal_connect(button, "toggled", cb, cb_data);
	
	return button;
}


void preferences()
{
	GtkWidget *hbox, *label, *button, *leftvbox, *rightvbox, *butbox;
	GtkBox *vbox;
	
	if (prefs_dialog) {
		gtk_window_present(GTK_WINDOW(prefs_dialog));
		return;
	}

	block_size = _blksize;
	opt = _opt;
	
	prefs_dialog = gtk_dialog_new_with_buttons(_("gpaco preferences"),
	 GTK_WINDOW(_window.window), GTK_DIALOG_DESTROY_WITH_PARENT, 
	 GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

	gtk_button_box_set_layout(
	 GTK_BUTTON_BOX(GTK_DIALOG(prefs_dialog)->action_area), GTK_BUTTONBOX_EDGE);
	
	gtk_window_set_resizable(GTK_WINDOW(prefs_dialog), FALSE); 
	gtk_container_set_border_width(GTK_CONTAINER(prefs_dialog), 8);

	vbox = GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox);
	gtk_box_set_spacing(vbox, 8);

	label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
	gtk_label_set_markup(GTK_LABEL(label), _("<span weight=\"bold\">Display</span>"));
	gtk_box_pack_start(vbox, label, FALSE, FALSE, 4);
	hbox = gtk_hbox_new(TRUE, 16);
	gtk_box_pack_start(vbox, hbox, FALSE, FALSE, 0);	
	leftvbox = gtk_vbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(hbox), leftvbox, TRUE, TRUE, 0);
	rightvbox = gtk_vbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(hbox), rightvbox, TRUE, TRUE, 0);	
	
	label = gtk_label_new(_("Size units:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
	gtk_box_pack_start(GTK_BOX(leftvbox), label, FALSE, FALSE, 0);
	butbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(leftvbox), butbox, FALSE, FALSE, 0);

	button = gp_radio_button(_("Human _readable"), NULL, (_blksize == HUMAN_READABLE),
	 butbox, G_CALLBACK(set_blksize), GINT_TO_POINTER(HUMAN_READABLE));
	gp_radio_button(_("_Bytes"), button, (_blksize == BYTE), butbox,
	 G_CALLBACK(set_blksize), GINT_TO_POINTER(BYTE));
	gp_radio_button(_("_Kilobytes"), button, (_blksize == KILOBYTE), butbox,
	 G_CALLBACK(set_blksize), GINT_TO_POINTER(KILOBYTE));
	gp_radio_button(_("_Megabytes"), button, (_blksize == MEGABYTE), butbox,
	 G_CALLBACK(set_blksize), GINT_TO_POINTER(MEGABYTE));
	 
	label = gtk_label_new(_("Date format:"));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
	gtk_box_pack_start(GTK_BOX(rightvbox), label, FALSE, FALSE, 0);
	butbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(rightvbox), butbox, FALSE, FALSE, 0);

	button = gp_radio_button(_("_Day"), NULL, ((_opt & SHOW_DAY) && !(_opt & SHOW_HOUR)),
	 butbox, G_CALLBACK(set_date_format), GINT_TO_POINTER(SHOW_DAY));
	gp_radio_button(_("Ho_ur"), button, (!_opt & SHOW_DAY && _opt & SHOW_HOUR),
	 butbox, G_CALLBACK(set_date_format), GINT_TO_POINTER(SHOW_HOUR));
	gp_radio_button(_("Day a_nd hour"), button, ((_opt & SHOW_DAY) && (_opt & SHOW_HOUR)),
	 butbox, G_CALLBACK(set_date_format), GINT_TO_POINTER(SHOW_DAY | SHOW_HOUR));
	
	button = gtk_check_button_new_with_mnemonic(_("Show missing sizes as _percent"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), (_opt & MISS_SIZE_AS_PERCENT));
	g_signal_connect(button, "toggled", G_CALLBACK(switch_percent), NULL);
	gtk_box_pack_start(vbox, button, FALSE, FALSE, 4);	
	
	g_signal_connect(prefs_dialog, "destroy", G_CALLBACK(pref_dialog_destroy), NULL);
	g_signal_connect(prefs_dialog, "response", G_CALLBACK(pref_response), NULL);
	  
	gtk_widget_show_all(prefs_dialog);
}


/* vim: set ts=4 */
