/*********************************************************************
 * properties.c - The "Properties" tab in the package window.
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2005 David Rosal <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/

#include "gpaco.h"


static gint istitle(gchar *p)
{
	guint i, n;
	gchar *titles[] = { "Name:", "Author:", "Version:", "Summary:", "URL:",
		"License:", "Description", "configure options", NULL };
	
	g_return_val_if_fail(p != NULL, 0);
	
	for (i = 0; i < g_strv_length(titles); i++) {
		n = strlen(titles[i]);
		if (!strncmp(p, titles[i], n))
			return (n + strspn(p + n, " "));
	}
	return 0;
}


GtkWidget *properties_page(pkg_t *pkg)
{
	GtkWidget *textview, *scroll, *vbox, *label;
	GtkTextBuffer *txtbuf;
	GtkTextIter end;
	GdkPixbuf *icon, *aux = NULL;
	gboolean info = FALSE;
	FILE *f;
	gchar buf[KILOBYTE << 2], *buf_8;
	gint n;

	g_return_val_if_fail(pkg != NULL, NULL);
	
	textview = gtk_text_view_new();
	scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
	  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scroll), textview);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(textview), FALSE);
    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textview), FALSE);
    g_object_set(textview, "left-margin", 10, "right-margin", 10, NULL);

	label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
	
	vbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	
	txtbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_create_tag(txtbuf, "wrap", "wrap-mode", GTK_WRAP_WORD_CHAR, NULL);
	gtk_text_buffer_create_tag(txtbuf, "bold", "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag(txtbuf, "console", "family", "courier", NULL);
	gtk_text_buffer_create_tag(txtbuf, "title", "size-points", 14.0, NULL);
	gtk_text_buffer_get_end_iter(txtbuf, &end);

	if (pkg->icon && (aux = gdk_pixbuf_new_from_file(pkg->icon, NULL))) {
		if (gdk_pixbuf_get_width(aux) > 72 || gdk_pixbuf_get_height(aux) > 72) {
		 	icon = gdk_pixbuf_scale_simple(aux, 72, 72, GDK_INTERP_HYPER);
			g_object_unref(aux);
		}
		else
			icon = aux;
		gtk_text_buffer_insert(txtbuf, &end, "\n", 1);
		gtk_text_buffer_insert_pixbuf(txtbuf, &end, icon);
		gtk_text_buffer_insert(txtbuf, &end, "\n", 1);
	}
	gtk_text_buffer_insert(txtbuf, &end, "\n", 1);
	
	if (pkg->info == NULL)
		goto Return;
	
	g_return_val_if_fail((f = g_fopen(pkg->info, "r")), vbox);
		
	while (paco_fgets(buf, sizeof(buf), f)) {
		if (buf[0] == '#')
			continue;
		buf_8 = gp_locale_to_utf8(buf);
		if ((n = istitle(buf_8)) > 0) {
			gtk_text_buffer_insert_with_tags_by_name(txtbuf, &end, buf_8, n, "bold", "console",
			  "title", NULL);
			gtk_text_buffer_insert_with_tags_by_name(txtbuf, &end, buf_8 + n, -1, "wrap", NULL);
		}
		else
			gtk_text_buffer_insert_with_tags_by_name(txtbuf, &end, buf_8, -1, "wrap", NULL);

		gtk_text_buffer_insert(txtbuf, &end, "\n", 1);
		g_free(buf_8);
		info = TRUE;
	}
	
	rewind(f);
	
	while (paco_fgets(buf, sizeof(buf), f)) {
		if (strncmp(buf, "#c:", 3))
			continue;
		buf_8 = gp_locale_to_utf8(buf);
		gtk_text_buffer_insert_with_tags_by_name(txtbuf, &end,
		 "\nconfigure options\n", -1, "bold", "console", "title", NULL);
		gtk_text_buffer_insert_with_tags_by_name(txtbuf, &end,
		 (buf_8[3] == '-')? (&buf_8[3]) : "(no options)", -1, "wrap", NULL);
		gtk_text_buffer_insert(txtbuf, &end, "\n", 1);
		g_free(buf_8);
		info = TRUE;
	}
	
	fclose(f);

	Return:
	if (info == FALSE)
		gtk_label_set_text(GTK_LABEL(label), _(" (no information available)"));

	return vbox;
}


/* vim: set ts=4 */
