/*********************************************************************
 * util.h
 *********************************************************************
 * This file is part of the package paco
 * Copyright (C) 2005 David Rosal i Ricart <david.rosal@upf.edu>
 * For more information visit http://paco.sourceforge.net
 *********************************************************************/
/* vim: set ts=4 */

#ifndef __PACO_UTIL_H__
#define __PACO_UTIL_H__

#include "config.h"

#ifdef __cplusplus
extern "C" {
#else
#	include <stdio.h>  /* FILE* */
#endif

#define PACO_STRING  0
#define PACO_INT     1
#define PACO_FLAGS   2

/* error codes */
#define PACO_ERROR_EMPTY_LOG  -1

/* Size units */
#define HUMAN_READABLE	-1
#define BYTE		 	1
#define KILOBYTE		1024
#define MEGABYTE		(1024 * 1024)

#define SIZE_SYMLINK	-1
#define SIZE_MISSING	-2

/* Do not allow log directories nor log files in these paths */
#define DEFAULT_EXCLUDE  "/dev:/proc"

/* Signals that cause the program to exit */
#define FATAL_SIGNALS  { SIGSEGV, SIGHUP, SIGINT, SIGQUIT, SIGABRT, SIGTERM, 0 }


#if !HAVE_STRCASECMP
extern int strcasecmp(const char *a, const char *b);
#endif
#if !HAVE_STRNCASECMP
extern int strncasecmp(const char *a, const char *b, int n);
#endif

#if !HAVE_STRERROR
#	define strerror(errno)  "error"
#endif

extern int paco_inpaths(const char *path, const char *list);
extern int paco_realpath(const char *path, char *real, int res_dir_symlinks);
extern char *paco_fgets(char *buf, int size, FILE *f);
extern int paco_getconf(const char *file, const char *tag, void *var, int type,
                        int flag);
extern void paco_die(const char *fmt, ...)
  __attribute__((__noreturn__,__format__(printf, 1, 2)));
extern void paco_die_errno(const char *fmt, ...)
  __attribute__((__noreturn__,__format__(printf, 1, 2)));
extern FILE *paco_fopen(const char *path, const char *mode);
extern void paco_rename(const char *oldpath, const char *newpath);
extern void paco_touch(const char *path);
extern int paco_update(const char *log, const char *tmp_log, int apparent);
extern long paco_getsize(int apparent, const char *fmt, ...)
  __attribute__((__format__(printf, 2, 3)));

#ifdef __cplusplus
}
#endif

#endif /* __PACO_UTIL_H__ */
